/*
 * @(#)TanButtonBar.java	1.00 98/08/28
 * 
 * Modification Log:
 *
 */

import java.awt.*;
import java.awt.event.*;

/**
 * Button bar. 
 * Similar to the flat cool buttons in Microsoft Office 
 * applications. Very light weight and easy to use.
 *
 * You are free to use this code and to make modifications provided
 * this notice is retained.
 * <p>
 * If you found this useful, please add a note of acknowledgement to my guestbook,
 * or send me a mail at <a href="mailto:tanmaykm@hotmail.com">tanmaykm@hotmail.com</a>. 
 * If you would like to report a bug or suggest some improvements, 
 * you are most welcome. I will be happy to help you use this piece of code.
 * <p>
 *
 * @author 	    Tanmay K. Mohapatra
 * @version     1.00, 28th July, 1998
 */

final public class TanButtonBar extends Canvas implements KeyListener, MouseListener, MouseMotionListener, FocusListener, ComponentListener
{
    String []   names;
    int []      aiXPosList, aiYPosList, aiWidthList;
    int         iActiveIndex, iHeight, iBorderWidth, iCurrButn;
    boolean     bIsPressed, bHasFocus, bHasMouse;
    Color       bgColor, fgColor;
    Font        font;
    Image       barImage;
    Insets      insets;
    Dimension   dPrefSize, dMinSize;
    Toolkit     tk;
    ActionListener actionListener;  // Post action events to listeners

    // dimensions used in drawing and scrolling
    Dimension   currSize, actSize, drawSize;
    int         iDispStart=0;
    int         iScrollBtnWidth;
    boolean     bScrollEnabled;

    /**
     * Constructor.
     */
    public TanButtonBar()
    {
        iBorderWidth = 10;
        iScrollBtnWidth = 5;
        addKeyListener(this);
        addMouseListener(this);
        addMouseMotionListener(this);
        addFocusListener(this);
        addComponentListener(this);
    }

    /**
     * Sets the button names. Button names appear in the same order as
     * in the string array. 
     * When you click on a button the action event name is the same as 
     * the button name.
     */
    public void setButtons(String [] namesList)
    {
        int iIndex;
        for (names = new String[iIndex=namesList.length], iIndex--; iIndex >= 0; names[iIndex] = new String(namesList[iIndex--]));
        barImage = null;
        aiWidthList = null;
        try
        {
            getBBSize(iBorderWidth);
        }
        catch (Exception e)
        {
            // component not yet added
        }
        iCurrButn = 0;
    }

    public boolean isFocusTraversable()
    {
        return true;
    }

    public void paint(Graphics g)
    {
        int iCurrStart;
        if (null != barImage)
        {
            // calculate the start position
            iCurrStart=0;
            if (iDispStart > 0)
            {
                iCurrStart = iBorderWidth/2;
                for (int iIndex=0; iIndex<iDispStart; iIndex++)
                {
                    iCurrStart += aiWidthList[iIndex] + iBorderWidth;
                }
            }
            g.drawImage(barImage, -iCurrStart, 0, this);
        }
        else
        {
            getParent().doLayout();
            updateImage();
            repaint();
            return;
        }

        if (bScrollEnabled)
        {
            // draw the scroll buttons
            g.setColor(getBackground());
            g.fillRect(currSize.width-iScrollBtnWidth, 0, iScrollBtnWidth, drawSize.height);
            g.fillRect(0, 0, iScrollBtnWidth, drawSize.height);
            int iSpillOver = currSize.width-drawSize.width+iCurrStart;
            if (iSpillOver > 0)
            {
                g.fillRect(currSize.width-iSpillOver, 0, iSpillOver, drawSize.height);
            }

            g.setColor(getForeground());
            int [] xPts = new int[3];
            int [] yPts = new int[3];
            xPts[0] = 0;                yPts[0] = currSize.height/2;
            xPts[1] = iScrollBtnWidth;  yPts[1] = currSize.height*3/4;
            xPts[2] = iScrollBtnWidth;  yPts[2] = currSize.height/4;
            g.fillPolygon(xPts, yPts, 3);
            xPts[0] = currSize.width;                   yPts[0] = currSize.height/2;
            xPts[1] = currSize.width-iScrollBtnWidth;   yPts[1] = currSize.height*3/4;
            xPts[2] = currSize.width-iScrollBtnWidth;   yPts[2] = currSize.height/4;
            g.fillPolygon(xPts, yPts, 3);
        }    
    }

    public void update(Graphics g)
    {
        if (null == barImage)
        {
            updateImage();
        }
        paint(g);
    }

    /**
     * Update the offscreen image.
     * Create a new one if size changes.
     */
    public void updateImage()
    {
        // create the off screen image
        currSize = getSize();
        actSize  = getBBSize(iBorderWidth);
        if (currSize.width > actSize.width)
        {
            bScrollEnabled = false;
            iDispStart = 0;
            drawSize = new Dimension(currSize.width, currSize.height);
        }
        else
        {
            bScrollEnabled = true;
            drawSize = new Dimension(actSize.width, currSize.height);
        }

        if ( (0 == drawSize.height) || (0 == drawSize.width) )
        {
            return;
        }

        if (null == barImage)
        {
            try
            {
                barImage = createImage(drawSize.width, drawSize.height);
            }
            catch (IllegalArgumentException iae)
            {
                return;  // ignore
            }
        }

        Graphics g = barImage.getGraphics();
        g.setColor(getBackground());
        g.fillRect(0, 0, drawSize.width, drawSize.height);

        // determine the font metrics and calculate the pixel positions
        // to print the button names and to place the highlight bar.
        FontMetrics fm       = tk.getFontMetrics(getFont());
        int         iHeight  = (drawSize.height + fm.getAscent())/2;
        int         iWidth   = iBorderWidth + iScrollBtnWidth;
        int         iHMargin = (iHeight - fm.getAscent())/2;

        if (bHasFocus || bHasMouse)
        {
            // highlight the current button
            int iCurrButnStart = iBorderWidth/2 + iScrollBtnWidth;
            for (int iIndex=0; iIndex<iCurrButn; iIndex++)
            {
                iCurrButnStart += aiWidthList[iIndex] + iBorderWidth;
            }
            g.fill3DRect(iCurrButnStart, iHMargin, aiWidthList[iCurrButn] + iBorderWidth, drawSize.height-2*iHMargin, !bIsPressed);
        }
        g.setColor(getForeground());

        if (null != names)
        {
            // draw the button names
            int         iIndex;
            for (iIndex=0; iIndex < names.length; iIndex++)
            {
                g.drawString(names[iIndex], iWidth, iHeight);
                iWidth += aiWidthList[iIndex] + iBorderWidth;
            }
        }
    }


    public synchronized Dimension preferredSize()
    {
        try
        {
            dPrefSize = getBBSize(iBorderWidth);
        }
        catch (Exception e)
        {
            dPrefSize=new Dimension(2000,20);
        }
        return (dPrefSize);
    }


    public Dimension getPreferredSize()
    {
        try
        {
            dPrefSize = getBBSize(iBorderWidth);
        }
        catch (Exception e)
        {
            dPrefSize=new Dimension(2000,20);
        }
        return (dPrefSize);
    }

    public Dimension getMinimumSize()
    {
        try
        {
            dMinSize = getBBSize(iBorderWidth);
        }
        catch (Exception e)
        {
            dMinSize=new Dimension(20,20);
        }
        return (dMinSize);
    }

    private void manageToGetToolkit()
    {
        if (null == (tk=getToolkit()))
        {
            tk = Toolkit.getDefaultToolkit();
        }
    }

    private Dimension getBBSize(int iBorderWidth)
    {
        Dimension   size;
        int         iIndex;

        manageToGetToolkit();
        size = new Dimension();

        if (null != aiWidthList)
        {
            size.width = iBorderWidth + 2*iScrollBtnWidth;
            for (iIndex=aiWidthList.length-1; iIndex >= 0; iIndex--)
            {
                size.width += aiWidthList[iIndex] + iBorderWidth;
            }
        }
        else if (null != names)
        {
            size.width = iBorderWidth + 2*iScrollBtnWidth;
            if (null == tk)
            {
                size.width += 10;
            }
            else
            {
                FontMetrics fm = tk.getFontMetrics(getFont());
                int         iNumButns;

                iNumButns = names.length;
                aiWidthList = new int[iNumButns];
                for (iIndex=0; iIndex < iNumButns; iIndex++)
                {
                    size.width += (aiWidthList[iIndex] = fm.stringWidth(names[iIndex])) + iBorderWidth;
                }
                iHeight = fm.getHeight() + iBorderWidth;
            }
        }

        size.height = (iHeight > 0) ? iHeight : 20;

        return size;
    }


    public void focusGained(FocusEvent fe)
    {
        bHasFocus = true;
        updateImage();
        repaint();
    }

    public void focusLost(FocusEvent fe)
    {
        bHasFocus = false;
        updateImage();
        repaint();
    }

    public void keyPressed(KeyEvent ke)
    {
        switch(ke.getKeyCode())
        {
            case KeyEvent.VK_RIGHT:
            case KeyEvent.VK_UP:
                if (iCurrButn < names.length-1)
                {
                    focusOnButton(iCurrButn+1);
                    if (bScrollEnabled)
                    {
                        int iReqWidth = iBorderWidth + iScrollBtnWidth;
                        int iIndex;
                        for (iIndex=iDispStart; iIndex<=iCurrButn; iIndex++)
                        {
                            iReqWidth += aiWidthList[iIndex] + iBorderWidth;
                        }

                        if(iReqWidth>currSize.width) scroll(false);
                    }
                }
                break;
            case KeyEvent.VK_LEFT:
            case KeyEvent.VK_DOWN:
                if (iCurrButn > 0)
                {
                    focusOnButton(iCurrButn-1);
                    if(bScrollEnabled && iCurrButn<iDispStart) scroll(true);
                }
                break;
            case KeyEvent.VK_ENTER:
            case KeyEvent.VK_SPACE:
                pressButton();
                break;
        }
    }

    public void keyReleased(KeyEvent ke)
    {
        if ( (ke.getKeyCode() == KeyEvent.VK_ENTER) || (ke.getKeyCode() == KeyEvent.VK_SPACE) )
        {
            releaseButton();
        }
    }

    public void keyTyped(KeyEvent ke)
    {
        if ( (ke.getKeyCode() == KeyEvent.VK_ENTER) || (ke.getKeyCode() == KeyEvent.VK_SPACE) )
        {
            actionButton();
        }
    }

    public void mouseClicked(MouseEvent me)
    {
    }

    public void mousePressed(MouseEvent me)
    {
        pressButton();
    }

    public void mouseReleased(MouseEvent me)
    {
        releaseButton();
    }

    public void mouseEntered(MouseEvent me)
    {
        bHasMouse = true;
        updateImage();
        repaint();
    }

    public void mouseExited(MouseEvent me)
    {
        bHasMouse = false;
        updateImage();
        repaint();
    }

    public void mouseDragged(MouseEvent me)
    {
    }

    public void mouseMoved(MouseEvent me)
    {
        focusOnButton(trackButton(me.getX()));
    }

    public void componentResized(ComponentEvent ce)
    {
        barImage = null;
        aiWidthList = null;
        updateImage();
        repaint();
    }

    public void componentMoved(ComponentEvent ce)
    {
    }

    public void componentShown(ComponentEvent ce)
    {
    }

    public void componentHidden(ComponentEvent ce)
    {
    }

  /**
   * Adds the specified action listener to receive action events
   * from this button.
   * @param listener the action listener
   */
   public void addActionListener(ActionListener listener)
   {
       actionListener = AWTEventMulticaster.add(actionListener, listener);
       enableEvents(AWTEvent.MOUSE_EVENT_MASK);
   }
 
   /**
    * Removes the specified action listener so it no longer receives
    * action events from this button.
    * @param listener the action listener
    */
   public void removeActionListener(ActionListener listener)
   {
       actionListener = AWTEventMulticaster.remove(actionListener, listener);
   }
 

    private void focusOnButton(int iIndex)
    {
        if (iCurrButn != iIndex)
        {
            iCurrButn = iIndex;
            updateImage();
            repaint();
        }
    }

    private void pressButton()
    {
        if (!bIsPressed)
        {
            bIsPressed = true;
            updateImage();
            repaint();
        }
    }

    private void releaseButton()
    {
        if (bIsPressed)
        {
            bIsPressed = false;
            updateImage();
            repaint();
            actionButton();
        }
    }

    private void scroll(boolean bToRight)
    {
        if(bToRight)
        {
            if(iDispStart > 0) iDispStart--;
        }
        else
        {
            if(iDispStart < (names.length-1)) iDispStart++;
        }
    }

    private void actionButton()
    {
        if(actionListener != null) 
        {
            actionListener.actionPerformed(new ActionEvent(this, ActionEvent.ACTION_PERFORMED, names[iCurrButn]));
        }
    }

    private int trackButton(int iXCoord)
    {
        if (bScrollEnabled)
        {
            if (iXCoord < iScrollBtnWidth)
            {
                scroll(true);   // scroll right
            }
            else if (iXCoord > (currSize.width-iScrollBtnWidth))
            {
                int iReqWidth = iBorderWidth + iScrollBtnWidth;
                int iIndex;
                for (iIndex=iDispStart; iIndex<names.length; iIndex++)
                {
                    iReqWidth += aiWidthList[iIndex] + iBorderWidth;
                }

                if(iReqWidth>currSize.width) scroll(false); // scroll left
            }
        }

        if (iDispStart > 0)
        {
            int iCurrStart = iBorderWidth/2;
            for (int iIndex=0; iIndex<iDispStart; iIndex++)
            {
                iCurrStart += aiWidthList[iIndex] + iBorderWidth;
            }
            iXCoord += iCurrStart;
        }

        int iIndex, iOffset, iThisWidth;
        iOffset = iBorderWidth/2;
        for (iIndex=0; iIndex < aiWidthList.length; iIndex++)
        {
            iThisWidth = aiWidthList[iIndex] + iBorderWidth;
            if ( (iXCoord > iOffset) && (iXCoord < iOffset+iThisWidth) )
            {
                return iIndex;
            }
            iOffset += iThisWidth;
        }
        return iCurrButn;
    }
}
